#include "DeadTime.h"
#include <iomanip>
#include <iostream>
#include <map>
#include <Random.h>
#include "errstr.h"

DeadTime::DeadTime(FrameList &frameList)
{
	_frameList = frameList;
}

void DeadTime::doCorr(LCGrid *lc)
{
	std::vector<double> deadTime, gtiTime;
	// Initialize _fracExp vector
	_deadTimeCorr =  new std::vector<double>;
	  for (unsigned int i = 0 ; i<=lc->_cells ; i++)
	  {
		  _deadTimeCorr->push_back(0.0);
		  deadTime.push_back(0.0);
		  gtiTime.push_back(0.0);
	  }

	unsigned int i = 0;  // GTI counter
	unsigned int n = 0;  // Frames counter
	double newDeadTime = 0.0;
	// Skip all the frames that starts earlier than the first GTI
	while ((_frameList._frameList[n].start() < (*(lc->_gtiList))[0].start()) &&
			( _frameList._frameList[n].stop() <= (*(lc->_gtiList))[0].stop() ) )
		n++;

	while( (i != (*(lc->_gtiList)).size()) &&  (n != _frameList._frames.size() ) )
	{
		errstr  << setprecision(19)<< "GTI[ "<<i<<" ]= [ "<< (*(lc->_gtiList))[i].start() << " , "
			<< (*(lc->_gtiList))[i].stop() << " ]"<< Sas::message(Msg::Lib, Msg::Verbose);
		double deadBinTime = 0.0;

		bool nextGTI = false;
		while ( (n != _frameList._frames.size() ) && ( nextGTI == false) )
		{
		  if (newDeadTime != 0.0)
		  {
		    n=n-1;
		    //deadBinTime = newDeadTime;
		    //newDeadTime = 0.0;
		  }
			errstr << "Frame [ " << n << " ]  = [ "<< _frameList._frameList[n].start() << " , "
				<< _frameList._frameList[n].stop() << " ]"<< Sas::message(Msg::Lib, Msg::Verbose);
			if ( (n == 0) && ( ( _frameList._frameList[n].start() > (*(lc->_gtiList))[i].start() )
							&& ( _frameList._frameList[n].start() <= (*(lc->_gtiList))[i].stop() )  ) )
			{
				errstr << "First Frame =[ "<< _frameList._frameList[n].start() << " , "
				<< _frameList._frameList[n].stop() << " ]" <<Sas::message(Msg::Lib, Msg::Verbose);
				deadBinTime = deadBinTime + (_frameList._frameList[n].start()-(*(lc->_gtiList))[i].start());

			}
			else if ( ( _frameList._frameList[n].start() > (*(lc->_gtiList))[i].start() )
					&& ( _frameList._frameList[n].start() <= (*(lc->_gtiList))[i].stop() ) )
			{
				if (  _frameList._frameList[n-1].stop() >= (*(lc->_gtiList))[i].start() )
				{
					deadBinTime = deadBinTime + ( _frameList._frameList[n].start()-_frameList._frameList[n-1].stop()  );

				}
				else
				{
					errstr << "First deadTime inside current bin =[ "<< _frameList._frameList[n].start() << " , "
						<< _frameList._frameList[n].stop() << " ]" <<Sas::message(Msg::Lib, Msg::Verbose);
					deadBinTime = deadBinTime + (_frameList._frameList[n].start()-(*(lc->_gtiList))[i].start());
					newDeadTime = 0.0;
					

				}
			}
			else if ( (n != 0) && ( _frameList._frameList[n].start() >= (*(lc->_gtiList))[i].stop() )
					&& ( ( _frameList._frameList[n-1].stop() >= (*(lc->_gtiList))[i].start() )
							&& ( _frameList._frameList[n-1].stop() < (*(lc->_gtiList))[i].stop() ) ) )
			{
				errstr << "Last deadTime inside current bin =[" << _frameList._frameList[n].start() << " , "
				<< _frameList._frameList[n].stop() << " ]" << Sas::message(Msg::Lib, Msg::Verbose);
				deadBinTime = deadBinTime + ( (*(lc->_gtiList))[i].stop()-_frameList._frameList[n-1].stop()  );
				newDeadTime = _frameList._frameList[n].start() - (*(lc->_gtiList))[i].stop();
			}

			if ( _frameList._frameList[n].stop() >= (*(lc->_gtiList))[i].stop() )
			{
				nextGTI = true;
			}

			n++;
		}

		int bin = lc->cell((((*(lc->_gtiList))[i].stop() + (*(lc->_gtiList))[i].start())/2.0));
		deadTime[bin] += deadBinTime;
		gtiTime[bin] += ((*(lc->_gtiList))[i].stop() - (*(lc->_gtiList))[i].start());
		i++;

	} // GTI while

	for (unsigned int i = 0 ; i < _deadTimeCorr->size() ; i++ )
	{
		if( gtiTime[i] != 0.0)
		{
			(*_deadTimeCorr)[i] = (*_deadTimeCorr)[i] +deadTime[i]/lc->getBinSize(i);
		}
		//_deadTimeCorr[i] +=   gtiTime[i] == 0.0 ? 0.0 : deadTime[i]/lc->getBinSize(i);
		deadTime[i] = 	deadTime[i] == 0.0 ? lc->getBinSize(i) : deadTime[i];
		lc->_fracExp[i] =  1.0 - deadTime[i]/lc->getBinSize(i);
		(*(lc->_effBinTime))[i] = (*(lc->_effBinTime))[i]-deadTime[i];
	}

	//	delete _deadTimeCorr;

}

void DeadTime::printCorr()
{
	for (unsigned int i = 0 ; i < _deadTimeCorr->size() ; i++ )
		errstr << "DeadTimeCorr["<<i<<"]= "<< (*_deadTimeCorr)[i] << Sas::message(Msg::Lib, Msg::Sparse);
}
